<?php
/*
Plugin Name: Интеграция Ninja Forms с amoCRM
Description: Гибкая интеграция Ninja Forms с amoCRM
Version: 1.0
Author: devamo.ru
Author URI: https://devamo.ru/ninjaforms
*/

if (!defined('ABSPATH')) {
    exit;
}

if (!class_exists('ninjaforms_amocrm')) {
    class ninjaforms_amocrm {
        
        const plugin_name = 'ninjaforms';
		const title = 'Настройка интеграции Ninja Forms с amoCRM';
		const menu_name = 'Ninja Forms amoCRM';
		const page_id = 'ninjaforms';
		const CF7V = 'CFV1';

        // Конструктор
        public function __construct() {
            add_action('init', [$this, 'save_utm_to_cookie']);
            add_action('admin_menu', [$this, 'add_admin_menu']);
            add_action('admin_bar_menu', [$this, 'add_admin_bar_menu'], 90 );
            add_action('admin_enqueue_scripts', [$this, 'add_scripts'], 11);
			add_action('ninja_forms_after_submission', [$this, 'send_complete'], 10, 2 );
			add_action('wp_ajax_export_'.SELF::plugin_name, [$this, 'export_lead']);
			add_action('wp_ajax_delete_lead_'.SELF::plugin_name, [$this, 'delete_lead']);
        }

		// отправляет в амо
		public function send_complete( $form ) {

			$form_id = $form['form_id'];
			$post_id = url_to_postid( isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : '');

			foreach($form['fields'] as $field){
				$form_data[$field['settings']['label']] = $field['value'];
			}
						
			if(!$form_data){
				return;
			}
    
			$form_fields = get_post_meta( $form_id, SELF::plugin_name.'-form', 1 );;
			if( isset($form_fields['status']) and $form_fields['status'] ){
				$settings = get_option(SELF::plugin_name.'-settings');
				if(!isset($settings['ym_counter'])){
					$settings = ['ym_counter' => ''];
				}
				date_default_timezone_set('UTC');
				$amocrm = get_option(SELF::plugin_name.'-amocrm');
				$field_types = $amocrm['field_types'];
				$utms = isset($amocrm['utm_fields']) ? $amocrm['utm_fields'] : false;
				$responsible_user_id = SELF::responsible_user_id($form_id);
				$lead_name = $form_fields['lead_name'];
				$note = $form_fields['note'];
				$task_text = $form_fields['task']['task_text'];
				$current_datetime = date('Y-m-d H:i:s');
		
				$page_name = get_the_title($post_id);
				$page_url = get_option('home').'?p='.$post_id;
		
				$form_name = SELF::get_form($form_id)['title'];
				$home_url = get_option('home');
				require (__DIR__ . '/fields_default.php');
		
				$ip = SELF::get_ip();
				$os = SELF::get_os($_SERVER['HTTP_USER_AGENT']);
				$browser = SELF::get_browser($_SERVER['HTTP_USER_AGENT']);   
				
				foreach($custom_tags as $custom_tag => $custom_tag_name){
					$lead_name = str_replace('{{'.$custom_tag.'}}', ${$custom_tag}, $lead_name);
					$note = str_replace('{{'.$custom_tag.'}}', ${$custom_tag}, $note);
					$custom_tag_value = $form_fields['custom_tags'][$custom_tag]; // !!!!!!!!!
			
					if($custom_tag_value !== '0' and $custom_tag_value !== ''){
						$_data = explode('-', $custom_tag_value);
						$entity = $_data['0'];
						if( $entity === '' or $entity === null ){
							continue;
						}
						$field_id = $_data['1'];
						$field_type = 'text';
						if(isset($field_types[$entity][$field_id])){
							$field_type = $field_types[$entity][$field_id];
						}
			
						if( in_array($field_type, ['date_time', 'birthday', 'date']) ){
							$form_value = strtotime(${$custom_tag});
						}elseif( in_array($field_type, ['checkbox']) ){
							$form_value = ${$custom_tag} ? true : false;
						}else{
							$form_value = (string)${$custom_tag};
						}
			
						if($form_value === '' or $form_value === null){
							continue;
						}
						
						if($field_id === 'price'){
							$form_value = (int)$form_value;
						}
			
						if( in_array($field_id, ['price', 'name', 'first_name', 'last_name']) ){
							$entities[$entity][$field_id] = $form_value;
						}else{
			
							if(is_array(${$custom_tag})){
								$values = [];
								foreach(${$custom_tag} as $fv){
									$values[] = ['value' => $fv];
								}
								$entities[$entity]['custom_fields_values'][] = [
									'field_id' => (int)$field_id,
									'values' => $values
								];
							}else{
								$entities[$entity]['custom_fields_values'][] = [
									'field_id' => (int)$field_id,
									'values' => [
										[
											'value' => $form_value
										]
									]
								];
							}
						}
					}
				}
		
				/*
				$form_data = $fields_data->get_posted_data();
				
				if( $attachments_data = $fields_data->uploaded_files() ){
					$names = array_keys($attachments_data);
					foreach($names as $name){
						if(isset($attachments_data[$name][0])){
							$path = $attachments_data[$name][0];
							$wp_upload_dir = wp_upload_dir();
							$upload_dir = $wp_upload_dir['basedir'].'/cf7_amocrm/';
							if(!is_dir($upload_dir)){
								mkdir($upload_dir, 0775, true);
							}
							$file_info = pathinfo($path);
							$new_file = $upload_dir.$file_info['basename'];
							if( !copy($path, $new_file) ){
								unset($form_data[$name]);
							}else{
								$form_data[$name] = $wp_upload_dir['baseurl'].'/cf7_amocrm/'.$file_info['basename'];
								$entities['files'][] = $wp_upload_dir['baseurl'].'/cf7_amocrm/'.$file_info['basename'];
							}
						}
					}
				}
				*/

				foreach($form_data as $form_field => $form_field_value){
					$key = isset($form_fields['fields'][$form_field]) ? $form_fields['fields'][$form_field] : false;
					if($key !== false and $key != '0'){
						$_data = explode('-', $key);
						$entity = $_data['0'];
						if( $entity === '' or $entity === null ){
							continue;
						}
						$field_id = $_data['1'];
			
						$field_type = 'text';
			
						if(isset($field_types[$entity][$field_id])){
							$field_type = $field_types[$entity][$field_id];
						}
		
						if( in_array($field_type, ['date_time', 'birthday', 'date']) ){
							$form_value = strtotime($form_field_value);
						}elseif( in_array($field_type, ['checkbox']) ){
							$form_value = $form_field_value ? true : false;
						}else{
							$form_value = (string)$form_field_value;
						}
			
						if($form_value === '' or $form_value === null){
							continue;
						}
		
						if($field_id === 'price'){
							$form_value = (int)$form_value;
						}
			
						if( in_array($field_id, ['price', 'name', 'first_name', 'last_name']) ){
							$entities[$entity][$field_id] = $form_value;
						}else{
							if( in_array($field_type, ['checkbox']) ){
								$entities[$entity]['custom_fields_values'][] = [
									'field_id' => (int)$field_id,
									'values' => [
										[
											'value' => $form_value
										]
									]
								];
							}elseif(is_array($form_field_value)){
								$values = [];
								foreach($form_field_value as $fv){
									$values[] = ['value' => $fv];
								}
								$entities[$entity]['custom_fields_values'][] = [
									'field_id' => (int)$field_id,
									'values' => $values
								];
							}else{
								$entities[$entity]['custom_fields_values'][] = [
									'field_id' => (int)$field_id,
									'values' => [
										[
											'value' => $form_value
										]
									]
								];
							}
						}
					}
		
					if(is_array($form_field_value)){
						$form_field_value = implode(', ', $form_field_value);
					}
		
					if( is_string($form_field_value)){
						$note = str_replace('{{'.$form_field.'}}', $form_field_value, $note);
						$lead_name = str_replace('{{'.$form_field.'}}', $form_field_value, $lead_name);     
						$task_text = str_replace('{{'.$form_field.'}}', $form_field_value, $task_text);
		
						if($form_fields['tags']){
							foreach($form_fields['tags'] as $key => $tag){
								$form_fields['tags'][$key] = str_replace('{{'.$form_field.'}}', $form_field_value, $tag);
							}
						}
		
					}
				}
			
				if($utms){
					$ym_counter_field_id = 0;
					foreach($utms as $field_id => $utm){
						if (isset($_COOKIE[$utm])) {
							$entities['lead']['custom_fields_values'][] = [
								'field_id' => (int)$field_id,
								'values' => [
									[
										'value' => (string)$_COOKIE[$utm]
									]
								]
							];
						}
						if($utm === '_ym_counter'){
							$ym_counter_field_id = $field_id;
						}
					}
		
					if( $ym_counter_field_id and $settings['ym_counter'] ){
						$entities['lead']['custom_fields_values'][] = [
							'field_id' => (int)$ym_counter_field_id,
							'values' => [
								[
									'value' => (string)$settings['ym_counter']
								]
							]
						];
					}
				}
		
				$entities['task'] = [];
				if( $form_fields['task']['task_status'] == 1 ){
					$task_type = $form_fields['task']['task_type'];
					
					foreach($custom_tags as $custom_tag => $custom_tag_name){
						$task_text = str_replace('{{'.$custom_tag.'}}', ${$custom_tag}, $task_text);
					}
		
					$task_complete_till = $form_fields['task']['task_complete_till'];
					$task_complete_till_times = $form_fields['task']['task_complete_till_times'];
					
					$task_complete_till = strtotime($current_datetime . ' +' . $task_complete_till . ' ' . $task_complete_till_times );
					$entities['task'] = [
						'responsible_user_id' => (int)$responsible_user_id,
						'task_type_id' => (int)$task_type,
						'text' => $task_text,
						'complete_till' => strtotime($current_datetime),
						'duration' => $task_complete_till - strtotime($current_datetime),
					];
				}
		
				$pipeline_id = (int)$form_fields['pipeline_id'];
				$p_status = $form_fields['p_status'];
				$price = $form_fields['price'];
		
				$entities['lead']['name'] = $lead_name;
		
				$unsortered = strpos($p_status, 'unsortered_');
		
				if($pipeline_id){
					$entities['lead']['pipeline_id'] = $pipeline_id;
				}
		
				// если разобранное
				$entities['lead']['status_id'] = (int)$p_status;
				$entities['lead']['pipeline_id'] = $pipeline_id;       
				$entities['lead']['responsible_user_id'] = (int)$responsible_user_id;
		
				if($price){
					$entities['lead']['price'] = (int)$price;
				}           
		
				if($form_fields['tags']){
					foreach($form_fields['tags'] as $tag){
						if(empty($tag)){
							continue;
						}
						foreach($custom_tags as $custom_tag => $custom_tag_name){
							$tag = str_replace('{{'.$custom_tag.'}}', ${$custom_tag}, $tag);
						}
						$entities['lead']['tags_to_add'][] = ['name' => (string)$tag];
					}
				}
		
				if(isset($entities['contact'])){
					$entities['contact']['responsible_user_id'] = (int)$responsible_user_id;
				}
		
				if(isset($entities['company'])){
					$entities['company']['responsible_user_id'] = (int)$responsible_user_id;
				}
		
				$entities['note'] = $note;
				$entities['ip'] = $ip;
		
				if(isset($form_fields['salesbot']) and (int)$form_fields['salesbot']['status'] and (int)$form_fields['salesbot']['id']){
					$entities['salesbot'] = $form_fields['salesbot']['id'];
				}
		
				// неразобранное
				if($unsortered !== false){
					$entities['lead']['request_id'] = (string)time();
					$entities['lead']['source_name'] = get_option('blogname');
					$entities['lead']['source_uid'] = md5(microtime(true).'devamo.ru');
					$entities['lead']['_embedded']['metadata'] = [ 
						'ip' => $ip,
						'form_id' => $form_id,
						'form_sent_at' => (int)time(),
						'form_name' => $form_name,
						'form_page' => $page_url,
						'referer' => $_SERVER['HTTP_REFERER'] ? $_SERVER['HTTP_REFERER'] : $_SERVER['HTTP_HOST'],
						'category' => 'forms',
					];
					if($pipeline_id){
						$entities['lead']['_embedded']['source']['external_id'] = implode('_', [SELF::plugin_name, $pipeline_id]);
						$entities['lead']['_embedded']['source']['type'] = 'widget';
					}
				}
				
				$data['action'] = SELF::CF7V;      
				$data['data'] = json_encode($entities);      
				$data['redirect'] = 'https://devamo.ru/'.SELF::plugin_name.'/ajax';
				$data['url'] = $_SERVER['HTTP_HOST'];
		
				$curl = curl_init();
				curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
				curl_setopt($curl, CURLOPT_URL, 'https://proxy.amoapps.store/redirect.php');
				curl_setopt($curl, CURLOPT_HEADER, false);
				curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
				curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($data));
				curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
				curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
				$output = curl_exec($curl);
				curl_close($curl);
		
				SELF::save_lead($form_id, $entities);
			}
		}

		public function delete_lead(){
			$post = $_POST;
			if(!isset($post['leads_id'])){
				echo json_encode(['status' => 0, 'message' => 'Нет данных']);
				wp_die();
			}
			$delete = [];
			foreach($post['leads_id'] as $lead_id){
				wp_delete_post($lead_id, true);
				$delete[] = $lead_id;
			}
			echo json_encode(['status' => 1, 'delete' => $delete]);
			wp_die();
		}

		public function export_lead(){
			$post = $_POST;
			if(!isset($post['lead_id'])){
				echo json_encode(['status' => 0, 'message' => 'Нет данных']);
				wp_die();
			}
			global $wpdb;
			$post_type = SELF::plugin_name.'-lead';
			$lead = $wpdb->get_row( "SELECT * FROM $wpdb->posts WHERE post_type = '$post_type' AND post_status = 'publish' AND ID = '".$post['lead_id']."'" );
		
			$curl = curl_init();
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_URL, 'https://proxy.amoapps.store/redirect.php');
			curl_setopt($curl, CURLOPT_HEADER, false);
			curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
			curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query([
				'data' => $lead->post_excerpt,
				'action' => SELF::CF7V,
				'url' => $_SERVER['HTTP_HOST'],
				'redirect' => 'https://devamo.ru/'.SELF::plugin_name.'/ajax',
			]));
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
			$output = curl_exec($curl);
			curl_close($curl);
			$decode = json_decode($output, true);
			echo $output;
			wp_die();
		}

		// сохраняет отправленные заявки
		public static function save_lead($form_id, $data){
			$form = get_post($form_id, 'ARRAY_A');
			$settings = get_option(SELF::plugin_name.'-amocrm');
			$text = [];
			foreach($data as $entity => $entity_value){
    
				$entity_keys = is_array($entity_value) ? array_keys($entity_value) : false;
				if($entity_keys){
					foreach($entity_keys as $entity_key){
		
						$value = $entity_value[$entity_key];
						$pipeline_id = isset($data[$entity]['pipeline_id']) ? $data[$entity]['pipeline_id'] : false;
		
						if( in_array($entity, ['lead', 'contact', 'company']) ){
							switch ($entity_key) {
								case 'name':
									$name = 'Название';
									$text[$entity][0] = $name.': '.$value;
									break;
								case 'responsible_user_id':
									$text[$entity][3] = 'Ответственный: '. (isset($settings['managers'][$value]) ? $settings['managers'][$value] : 'Не выбран');
									break;
								case 'pipeline_id':
									$text[$entity][1] = 'Воронка: '.$settings['pipelines'][$pipeline_id]['name'];
									break;
								case 'status_id':
									if( isset($settings['pipelines'][$pipeline_id]['statuses'][$value]['name']) ){
										$text[$entity][2] = 'Этап воронки: '.$settings['pipelines'][$pipeline_id]['statuses'][$value]['name'];
									}
									break;
								case 'price':
									$text[$entity][4] = 'Бюджет: '.$value;
									break;
								case 'first_name':
									$text[$entity][0] = 'Имя: '.$value;
									break;
								case 'last_name':
									$text[$entity][1] = 'Фамилия: '.$value;
									break;
								case 'tags_to_add':
									if(isset($data['lead']['tags_to_add'])){
										$tags = [];
										foreach($data['lead']['tags_to_add'] as $tag){
											$tags[] = $tag['name'];
										}
										$text[$entity][99] = 'Теги: '.implode(', ', $tags);
									}
									break;
								case 'custom_fields_values':
									$fields = isset($data[$entity]['custom_fields_values']) ? $data[$entity]['custom_fields_values'] : false;
									if($fields){
										$i = 10;
										foreach($fields as $field){
											$field_name = false;
											$i++;
											if(isset($settings['fields'][$entity.'-'.$field['field_id']])){
												$field_name = $settings['fields'][$entity.'-'.$field['field_id']];
												$field_name =  str_replace(['Сделка :: ', 'Контакт :: ', 'Компания :: '], '', $field_name);
												if($field['values']){
													$field_values = [];
													foreach($field['values'] as $field_value){
														$field_values[] = $field_value['value'];
													}
													$text[$entity][$i] = $field_name.': '.implode(', ', $field_values);
												}
											}
				
											if(isset($settings['utm_fields'][$field['field_id']])){
												$field_name = $settings['utm_fields'][$field['field_id']];
												if($field['values']){
													$field_values = [];
													foreach($field['values'] as $field_value){
														$field_values[] = $field_value['value'];
													}
													$text[$entity][$i] = $field_name.': '.implode(', ', $field_values);
												}
											}
				
										}
									}
									break;
							}
							}
							
							if($entity === 'task'){
								$task = $data['task'];
								switch ($entity_key) {
									case 'responsible_user_id':
										if( isset($settings['managers'][$task['responsible_user_id']]) ){
											$text[$entity][3] = 'Ответственный: '.$settings['managers'][$task['responsible_user_id']];
										}
										break;
									case 'text':
										$text[$entity][0] = 'Описание: '.$task['text'];
										break;
									case 'complete_till':
										$text[$entity][1] = 'Дедлайн: '.date('d.m.Y H:i:s', $task['complete_till']);
										break;
									case 'task_type_id':
										$task_type_name = '';
										foreach($settings['task_types'] as $task_type){
											if($task['task_type_id'] == $task_type['id']){
												$text[$entity][2] = 'Тип: '.$task_type['name'];
											}
										}
										break;
								}
							}
						}
				}
		
				if($entity === 'note'){
					$text[$entity] = $entity_value;
				}           
				
			}

			$print = '';

			if(isset($text['lead'])){
				ksort($text['lead']);
				$print .= '<div>';
				$print .= '<div><b>Сделка</b></div>';
				$print .= '<p>'.implode('</p><p>', $text['lead']).'</p>';
				$print .= '</div><br>';
			}
		
			if(isset($text['contact'])){
				ksort($text['contact']);
				$print .= '<div>';
				$print .= '<div><b>Контакт</b></div>';
				$print .= '<p>'.implode('</p><p>', $text['contact']).'</p>';
				$print .= '</div><br>';
			}

			if(isset($text['company'])){
				ksort($text['company']);
				$print .= '<div>';
				$print .= '<div><b>Компания</b></div>';
				$print .= '<p>'.implode('</p><p>', $text['company']).'</p>';
				$print .= '</div><br>';
			}
		
			if(isset($text['task'])){
				ksort($text['task']);
				$print .= '<div>';
				$print .= '<div><b>Задача</b></div>';
				$print .= '<p>'.implode('</p><p>', $text['task']).'</p>';
				$print .= '</div><br>';
			}

			if(isset($text['note'])){
				$print .= '<div>';
				$print .= '<div><b>Примечание</b></div>';
				$print .= '<p>' . str_replace(PHP_EOL, '</p><p>', $text['note']) . '</p>';
				$print .= '</div>';
			}
			
			wp_insert_post(wp_slash([
				'post_title'    => date('d.m.Y H:i:s'),
				'post_parent'   => $form_id,
				'post_content'  => $print,
				'post_status'   => 'publish',
				'post_type'     => SELF::plugin_name.'-lead',
				'post_excerpt' => json_encode($data)
			]));
			
		}

		// получает браузер
		function get_browser($userAgent) {
			$browsers = [
			  'Firefox',
			  'Opera',
			  'Chrome',
			  'MSIE',
			  'Safari'
			];
			foreach($browsers as $browser){
				if(strpos($userAgent, $browser) !== false) { 
					return $browser;
				}
			}
			return 'Неизвестный';
		}

		// получает OS
		function get_os($userAgent) { 
			$os_platform  = "Unknown OS Platform";
			$os_array     =  [
				'/windows nt 10/i'      =>  'Windows 10',
				'/windows nt 6.3/i'     =>  'Windows 8.1',
				'/windows nt 6.2/i'     =>  'Windows 8',
				'/windows nt 6.1/i'     =>  'Windows 7',
				'/windows nt 6.0/i'     =>  'Windows Vista',
				'/windows nt 5.2/i'     =>  'Windows Server 2003/XP x64',
				'/windows nt 5.1/i'     =>  'Windows XP',
				'/windows xp/i'         =>  'Windows XP',
				'/windows nt 5.0/i'     =>  'Windows 2000',
				'/windows me/i'         =>  'Windows ME',
				'/win98/i'              =>  'Windows 98',
				'/win95/i'              =>  'Windows 95',
				'/win16/i'              =>  'Windows 3.11',
				'/macintosh|mac os x/i' =>  'Mac OS X',
				'/mac_powerpc/i'        =>  'Mac OS 9',
				'/linux/i'              =>  'Linux',
				'/ubuntu/i'             =>  'Ubuntu',
				'/iphone/i'             =>  'iPhone',
				'/ipod/i'               =>  'iPod',
				'/ipad/i'               =>  'iPad',
				'/android/i'            =>  'Android',
				'/blackberry/i'         =>  'BlackBerry',
				'/webos/i'              =>  'Mobile'
			];
			foreach ($os_array as $regex => $value){
				if (preg_match($regex, $userAgent)){
					$os_platform = $value;
				}
			}
			return $os_platform;
		}

		// получает ip
		public static function get_ip(){
			$client  = isset($_SERVER['HTTP_CLIENT_IP']) ? $_SERVER['HTTP_CLIENT_IP'] : false;
			$forward = isset($_SERVER['HTTP_X_FORWARDED_FOR']) ? $_SERVER['HTTP_X_FORWARDED_FOR'] : false;
			$remote  = isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : $_SERVER['REMOTE_ADDR'];
			if(filter_var($client, FILTER_VALIDATE_IP)){
				$ip = $client;
			}elseif(filter_var($forward, FILTER_VALIDATE_IP)){
				$ip = $forward;
			}else{
				$ip = $remote;
			}
			return $ip;
		}

		// ответственный
		public static function responsible_user_id($form_id){
			$form_fields = get_post_meta( $form_id, SELF::plugin_name.'-form', 1 );
			if( !isset($form_fields['responsible_list']) ){
				return 0;
			}
		
			$responsible_list = $form_fields['responsible_list'];
			$form_fields['current_responsible'] = isset($form_fields['current_responsible']) ? (int)$form_fields['current_responsible'] : 0;
			
			$form_fields['current_responsible']++;
			$form_fields['current_responsible'] = isset($responsible_list[($form_fields['current_responsible'])]) ? ($form_fields['current_responsible']) : 0;
			
			update_post_meta( $form_id, SELF::plugin_name.'-form', $form_fields );
			return (int)$responsible_list[($form_fields['current_responsible'])];
		}
		
		// Страница настроек
        public function plugin_page(){
			// сохраняем настройки формы
			if( isset($_POST['action']) and $_POST['action'] === 'save_form' and isset($_POST['nonce']) and wp_verify_nonce( $_POST['nonce'], SELF::plugin_name.'-form') ){
				$_POST[SELF::plugin_name.'-form']['tags'] = isset($_POST[SELF::plugin_name.'-form']['tags']) ? $_POST[SELF::plugin_name.'-form']['tags'] : [];
				update_post_meta( $_POST['form_id'], SELF::plugin_name.'-form', ($_POST[SELF::plugin_name.'-form']) );
			}

			// сохранем общие настройки плагина
			if( isset($_POST['action']) and $_POST['action'] === 'save_settings' and isset($_POST['nonce']) and wp_verify_nonce( $_POST['nonce'], SELF::plugin_name.'-settings_page') ){
				update_option( SELF::plugin_name.'-settings', $_POST[SELF::plugin_name.'-settings'], 'no' );
			}

			// обновляем данные из amoCRM
			if( isset($_GET['clear']) and $_GET['clear']){
				SELF::get_amo_fields(1);
			}

			require_once (__DIR__ . '/fields_default.php');
			$settings = get_option(SELF::plugin_name.'-settings');
			if(!$settings){
				$settings = [];
			}

			if( !isset($settings['ym_counter']) ){
				$settings['ym_counter'] = '';
			}

			$amocrm = get_option(SELF::plugin_name.'-amocrm');
			if( !$amocrm ){
				$amocrm = SELF::get_amo_fields();
			}

			$amoFields = isset($amocrm['fields']) ? $amocrm['fields'] : ['0' => 'Поля не загружены'];
			$amoManagers = isset($amocrm['managers']) ? $amocrm['managers'] : ['0' => 'Менеджеры не загружены'];

			if( isset($_GET['form_id']) and $_GET['form_id'] ){
				$get_post_meta = get_post_meta( $_GET['form_id'], SELF::plugin_name.'-form', 1 );
				$fields = $get_post_meta ? ($get_post_meta) : [];
				$all_tags = array_keys(array_merge($custom_tags, ['country' => 'Страна', 'region' => 'Регион', 'city' => 'Город']));
				$all_tags[] = 'order_id';
				$form_fields = SELF::get_form_fields($_GET['form_id']);
				if($form_fields){
					foreach($form_fields as $form_field){
						$all_tags[] = $form_field;
					}
				}

				$form = SELF::get_form($_GET['form_id']);

				$amoPipelines = isset($amocrm['pipelines']) ? $amocrm['pipelines'] : ['0' => ['name' => 'Воронки не загружены']];
				$fields['task'] = isset($fields['status']) ? $fields['task'] : $task_default;
				$task_types = $amocrm['task_types'];
				$note = isset($fields['note']) ? $fields['note'] : $note_default;
				$fields['status'] = isset($fields['status']) ? $fields['status'] : true;
				$fields['salesbot'] = isset($fields['salesbot']) ? $fields['salesbot'] : ['status' => 0, 'id' => 0];
				$fields['lead_name'] = isset($fields['lead_name']) ? $fields['lead_name'] : $lead_name_default;
				$fields['price'] = isset($fields['price']) ? $fields['price'] : 1000;
				$fields['pipeline_id'] = (!isset($fields['pipeline_id']) and $amoPipelines ) ? key($amoPipelines) : $fields['pipeline_id']; 
				$fields['tags'] = isset($fields['tags']) ? $fields['tags'] : $tags_default;
				$fields['responsible_list'] = !isset($fields['responsible_list'])  ? [] : $fields['responsible_list'];
				$title = 'Настройка формы :: ' . $form['title'];
				$bots = $amocrm['bots'];
				require_once (__DIR__ . '/templates/form_edit.php');
			}elseif( isset($_GET['lead_id']) and $_GET['lead_id'] ){
				$lead = get_post($_GET['lead_id'], 'ARRAY_A');
				$title = 'Заявка # '.$lead['ID'];
				require_once (__DIR__ . '/templates/lead_view.php');
			}elseif( isset($_GET['changelog']) ){
				$title = 'Лог изменений';
				require_once (__DIR__ . '/templates/changelog.php');
			}elseif( isset($_GET['debug']) ){
				$log = file_get_contents('https://devamo.ru/logs?widget=ninjaforms&url='.$_SERVER['SERVER_NAME']);
				$title = 'Отладка';
				require_once (__DIR__ . '/templates/debug.php');
			}else{
				$forms = SELF::get_forms();
				$leads = SELF::get_leads();
				$title = 'Общие настройки';
				require_once (__DIR__ . '/templates/settings_page.php');
			}
        }	
		
		// Формы
		public static function get_forms(){
			$forms = [];
			$Ninja_Forms = Ninja_Forms()->form()->get_forms();
			foreach($Ninja_Forms as $form){
				$forms[] = [
					'ID' => $form->get_id(),
					'post_title' => $form->get_setting( 'title' ),
					'post_date' => date('d.m.Y H:i', strtotime($form->get_setting( 'created_at' )))
				];
			}
			return $forms;
		}

		// Лид
		public static function get_leads(){
			global $wpdb;
			$post_type = SELF::plugin_name.'-lead';
			$forms = $wpdb->get_results( "SELECT * FROM $wpdb->posts WHERE post_type = '$post_type' AND post_status = 'publish' ORDER BY ID DESC LIMIT 100", 'ARRAY_A' );
			if($forms){
				foreach ( $forms as $key => $form ) {
					$forms[$key]['post_date'] = date('d.m.Y H:i', strtotime($form['post_date']));
				}
			}
			return $forms;
		}

		// Кол-во лидов формы
		public static function get_leads_count($form_id){
			global $wpdb;
			$post_type = SELF::plugin_name.'-lead';
			return $wpdb->get_var( "SELECT COUNT(*) FROM $wpdb->posts WHERE post_type = '$post_type' AND post_status = 'publish' AND post_parent = '$form_id'" );
		}
		
		// Форма
		public static function get_form($form_id){
			return ['title' => Ninja_Forms()->form($form_id)->get()->get_setting( 'title' ), 'ID' => Ninja_Forms()->form($form_id)->get()->get_id()];  
		}

		// Поля формы
		public static function get_form_fields($form_id){
			$fields = [];
			$form_fields = Ninja_Forms()->form($form_id)->get_fields();
			foreach($form_fields as $form_field){
				if( !in_array($form_field->get_setting( 'type' ), ['submit'])){
					$fields[$form_field->get_setting('label')] = $form_field->get_setting('label');
				}
			}
			return $fields;  
		}

        // utm метки
        public function save_utm_to_cookie() {
            $date = time()+3600*24*30;
            $options = get_option(SELF::plugin_name);
            $utms = isset($options['utm_fields']) ? $options['utm_fields'] : false;
            if($utms){
                foreach($utms as $utm){
                    if(isset($_GET[$utm])) setcookie($utm, $_GET[$utm], $date, "/"); 
                }
            }
        }
             
        // Меню в настройках
        public function add_admin_menu() {
            add_options_page(
                SELF::title, // page title 
                SELF::menu_name, 
                'manage_options', SELF::page_id.'_amocrm', 
                [$this, 'plugin_page']
            );
        }

        // Верхнее меню
        public function add_admin_bar_menu( $wp_admin_bar ) {
            $wp_admin_bar->add_menu( array(
                'id'    => SELF::page_id.'_amocrm',
                'title' => '<span class="ab-label">'.SELF::menu_name.'</span>',
                'href'  => '/wp-admin/options-general.php?page='.SELF::page_id.'_amocrm',
            ) );
        }
        
        // Получение данных amoCRM
        public static function get_amo_fields($clear_cache = 0){
            $data = [
                'action' => 'getAmoCF', 
                'url' => $_SERVER['HTTP_HOST'], 
                'companies' => true, 
                'clear' => $clear_cache,
                'redirect' => 'https://devamo.ru/'.SELF::plugin_name.'/ajax',
            ];
            $curl = curl_init();
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_URL, 'https://proxy.amoapps.store/redirect.php');
            curl_setopt($curl, CURLOPT_HEADER, false);
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($data));
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
            $json = curl_exec($curl);
            curl_close($curl);
            $output = json_decode($json, true);
        	if(isset($output['status']) and $output['status']){
                update_option( SELF::plugin_name.'-amocrm', $output['settings'] );
        	}
        	return isset($output['settings']) ? $output['settings'] : false;
        }

        // Активации
        public static function activate() {
            if (!function_exists('curl_version')) {
                deactivate_plugins(plugin_basename(__FILE__), true);
                wp_die('Curl не установлен! Для работы плагина требуется библиотека Curl, обратитесь в техническую поддержку хостинга или к вашему администратору сервера!');
            }

			if (!class_exists('Ninja_Forms')) {
                deactivate_plugins(plugin_basename(__FILE__), true);
                wp_die('Плагин Ninja Forms не установлен, ваши формы скорее всего созданы другим плагином');
			}
        
            if( !SELF::get_amo_fields() ){
                deactivate_plugins(plugin_basename(__FILE__), true);
                wp_die('Сайт '.$_SERVER['HTTP_HOST'].' не добавлен в настройках виджета amoCRM! Перейдите в amoCRM, выберите амоМаркет в левом меню > Установленые, перейдите в настройки виджета кликнув на него, в появившемся окне добавьте свой сайт. Техническая поддержка amo@devamo.ru или в telegram @bmite' );
            }
        }

        // Деактивации
        public static function deactivate() {
            delete_option(SELF::plugin_name.'-amocrm');
        }

        // Скрипты и стили
        public function add_scripts( $hook_suffix ) {
            if ( strpos( $hook_suffix, SELF::page_id.'_amocrm' ) === false ) {
                return;
            }
            wp_enqueue_script(SELF::plugin_name.'-chosen-js', plugins_url( '/'.SELF::plugin_name.'_amocrm/js/chosen.jquery.min.js' ), '', filemtime( __DIR__ . ('/js/chosen.jquery.min.js') ) ,true);
            wp_enqueue_script(SELF::plugin_name.'-js', plugins_url( '/'.SELF::plugin_name.'_amocrm/js/main.js' ), '', filemtime( __DIR__ . ('/js/main.js') ) ,true);
            
            wp_enqueue_style( SELF::plugin_name.'-chosen-css', plugins_url( '/'.SELF::plugin_name.'_amocrm/css/chosen.min.css' ), null, filemtime( __DIR__ . ('/css/chosen.min.css') ), 'all' );
            wp_enqueue_style( SELF::plugin_name.'-css', plugins_url( '/'.SELF::plugin_name.'_amocrm/css/main.css' ), null, filemtime( __DIR__ . ('/css/main.css') ), 'all' );
        
        }
    }

    // Инициализация плагина
    $ninjaforms_amocrm_instance = new ninjaforms_amocrm();

    // Регистрация хуков активации и деактивации
    register_activation_hook(__FILE__, ['ninjaforms_amocrm', 'activate']);
    register_deactivation_hook(__FILE__, ['ninjaforms_amocrm', 'deactivate']);
}
